#include <stdio.h>
#include <malloc.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <time.h>
#include <locale.h>
#include <alsa/asoundlib.h>
#include <assert.h>
#include <sys/poll.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <sys/signal.h>
#include <asm/byteorder.h>

snd_pcm_t *playback_handle, *capture_handle;

static struct {
        snd_pcm_format_t format;
        unsigned int channels;
        unsigned int rate;
} hwparams;

typedef long long off64_t;

static int fd[2];
static u_char *play_buf = NULL;
static u_char *cap_buf = NULL;
static const int test_nowait = 0;
static const char *pcm_name = "default";

static size_t chunk_bytes;
static snd_pcm_uframes_t chunk_size = 0;
static size_t bits_per_sample, bits_per_frame;
static unsigned period_time = 0;
static unsigned buffer_time = 0;
static snd_pcm_uframes_t period_frames = 0;
static snd_pcm_uframes_t buffer_frames = 0;

static int avail_min = -1;
static int start_delay = 0;
static int stop_delay = 0;

/*
 * Safe read (for pipes)
 */
static ssize_t safe_read(int fd, void *buf, size_t count)
{
        ssize_t result = 0, res;

        while (count > 0) {
                if ((res = read(fd, buf, count)) == 0)
                        break;
                if (res < 0)
                        return result > 0 ? result : res;
                count -= res;
                result += res;
                buf = (char *)buf + res;
        }
        return result;
}

static void set_params(snd_pcm_t *handle)
{
	snd_pcm_hw_params_t *params;
	snd_pcm_sw_params_t *swparams;
	snd_pcm_uframes_t buffer_size;
	int err;
	size_t n;
	unsigned int rate;
	snd_pcm_uframes_t start_threshold, stop_threshold;

	snd_pcm_hw_params_alloca(&params);
	snd_pcm_sw_params_alloca(&swparams);

	err = snd_pcm_hw_params_any(handle, params);
	if (err < 0) {
		fprintf(stderr, "Broken configuration for this PCM: no configurations available");
		exit(EXIT_FAILURE);
	}

	err = snd_pcm_hw_params_set_access(handle, params, SND_PCM_ACCESS_RW_INTERLEAVED);
	if (err < 0) {
		fprintf(stderr, "Access type not available");
		exit(EXIT_FAILURE);
	}

	err = snd_pcm_hw_params_set_format(handle, params, hwparams.format);
	if (err < 0) {
		fprintf(stderr, "Sample format non available");
		exit(EXIT_FAILURE);
	}

	err = snd_pcm_hw_params_set_channels(handle, params, hwparams.channels);
	if (err < 0) {
		fprintf(stderr, "Channels count non available");
		exit(EXIT_FAILURE);
	}

#if 0
	err = snd_pcm_hw_params_set_periods_min(handle, params, 2);
	assert(err >= 0);
#endif

	err = snd_pcm_hw_params_set_rate_near(handle, params, &hwparams.rate, 0);
	assert(err >= 0);

	rate = hwparams.rate;

	if (buffer_time == 0 && buffer_frames == 0) {
		err = snd_pcm_hw_params_get_buffer_time_max(params,
							    &buffer_time, 0);
		assert(err >= 0);
		if (buffer_time > 500000)
			buffer_time = 500000;
	}

	if (period_time == 0 && period_frames == 0) {
		if (buffer_time > 0)
			period_time = buffer_time / 4;
		else
			period_frames = buffer_frames / 4;
	}

	if (period_time > 0)
		err = snd_pcm_hw_params_set_period_time_near(handle, params,
							     &period_time, 0);
	else
		err = snd_pcm_hw_params_set_period_size_near(handle, params, &period_frames, 0);
	assert(err >= 0);

	if (buffer_time > 0) {
		err = snd_pcm_hw_params_set_buffer_time_near(handle, params,
							     &buffer_time, 0);
	} else {
		err = snd_pcm_hw_params_set_buffer_size_near(handle, params,
							     &buffer_frames);
	}
	assert(err >= 0);

	err = snd_pcm_hw_params(handle, params);
	if (err < 0) {
		fprintf(stderr, "Unable to install hw params:");
		exit(EXIT_FAILURE);
	}

	snd_pcm_hw_params_get_period_size(params, &chunk_size, 0);
	snd_pcm_hw_params_get_buffer_size(params, &buffer_size);
	if (chunk_size == buffer_size) {
		fprintf(stderr, "Can't use period equal to buffer size (%lu == %lu)",
		      chunk_size, buffer_size);
		exit(EXIT_FAILURE);
	}
	snd_pcm_sw_params_current(handle, swparams);
	if (avail_min < 0)
		n = chunk_size;
	else
		n = (double) rate * avail_min / 1000000;
	err = snd_pcm_sw_params_set_avail_min(handle, swparams, n);

	/* round up to closest transfer boundary */
	n = buffer_size;
	if (start_delay <= 0) {
		start_threshold = n + (double) rate * start_delay / 1000000;
	} else
		start_threshold = (double) rate * start_delay / 1000000;
	if (start_threshold < 1)
		start_threshold = 1;
	if (start_threshold > n)
		start_threshold = n;
	err = snd_pcm_sw_params_set_start_threshold(handle, swparams, start_threshold);
	assert(err >= 0);
	if (stop_delay <= 0) 
		stop_threshold = buffer_size + (double) rate * stop_delay / 1000000;
	else
		stop_threshold = (double) rate * stop_delay / 1000000;
	err = snd_pcm_sw_params_set_stop_threshold(handle, swparams, stop_threshold);
	assert(err >= 0);

	if (snd_pcm_sw_params(handle, swparams) < 0) {
		fprintf(stderr, "unable to install sw params:");
		exit(EXIT_FAILURE);
	}

	bits_per_sample = snd_pcm_format_physical_width(hwparams.format);
	bits_per_frame = bits_per_sample * hwparams.channels;
	chunk_bytes = chunk_size * bits_per_frame / 8;

	// fprintf(stderr, "real chunk_size = %i, frags = %i, total = %i\n", chunk_size, setup.buf.block.frags, setup.buf.block.frags * chunk_size);

	buffer_frames = buffer_size;	/* for position test */
}

/*
 *  write function
 */

static ssize_t pcm_write(snd_pcm_t * handle, u_char *data, size_t count)
{
        ssize_t r;
        ssize_t result = 0;

        if (count < chunk_size) {
                snd_pcm_format_set_silence(hwparams.format, data + count * bits_per_frame / 8, (chunk_size - count) * hwparams.channels);
                count = chunk_size;
        }
        while (count > 0) {
//                if (test_position)
//                        do_test_position();
//                r = writei_func(handle, data, count);
		r = snd_pcm_writei(handle, data, count);
//                if (test_position)
//                        do_test_position();
                if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
                        if (!test_nowait)
                                snd_pcm_wait(handle, 1000);
//                } else if (r == -EPIPE) {
//                        xrun();
//                } else if (r == -ESTRPIPE) {
//                        suspend();
                } else if (r < 0) {
                        fprintf(stderr, "write error: %s"), snd_strerror(r);
                        exit(EXIT_FAILURE);
                }
                if (r > 0) {
//                        if (vumeter)
//                                compute_max_peak(data, r * hwparams.channels);
                        result += r;
                        count -= r;
                        data += r * bits_per_frame / 8;
                }
        }
        return result;
}

/*
 *  read function
 */


static ssize_t pcm_read(snd_pcm_t * handle, u_char *data, size_t rcount)
{
        ssize_t r;
        size_t result = 0;
        size_t count = rcount;

        if (count != chunk_size) {
                count = chunk_size;
        }

        while (count > 0) {
//                if (test_position)
//                        do_test_position();
//                r = readi_func(handle, data, count);
		r = snd_pcm_readi(handle, data, count);
//                if (test_position)
//                        do_test_position();
                if (r == -EAGAIN || (r >= 0 && (size_t)r < count)) {
                        if (!test_nowait)
                                snd_pcm_wait(handle, 1000);
//                } else if (r == -EPIPE) {
//                        xrun();
//                } else if (r == -ESTRPIPE) {
//                        suspend();
                } else if (r < 0) {
                        fprintf(stderr, "read error: %s"), snd_strerror(r);
                        exit(EXIT_FAILURE);
                }
                if (r > 0) {
//                        if (vumeter)
//                                compute_max_peak(data, r * hwparams.channels);
                        result += r;
                        count -= r;
                        data += r * bits_per_frame / 8;
                }
        }
        return rcount;
}

/* playing raw data */

static void playback(char *name)
{
	size_t c, f;

        if ((fd[0] = open(name, O_RDONLY, 0)) == -1) {
                perror(name);
                exit (1);
        }

	while (1) {//直到读完音频文件为止。
		c = chunk_bytes;

		if(safe_read(fd[0], play_buf, c) != c)//读取文件
			break;

		f = c * 8 / bits_per_frame;
		if(pcm_write(playback_handle, play_buf, f) != f)
			break;
	}
	snd_pcm_drain(playback_handle);
}

static void capture(char *name)
{
	size_t c, f;

	/* open a file to write */
	if ((fd[1] = open64(name, O_WRONLY, 0644)) == -1) {
		perror(name);
		exit(EXIT_FAILURE);
	}

	/* capture */
	while (1) {
		c =chunk_bytes;
		f = c * 8 / bits_per_frame;
		if(pcm_read(capture_handle, cap_buf, f) != f)
			break;

                if (write(fd[1], cap_buf, c) != c) {
                	perror(name);
                        exit(EXIT_FAILURE);
                }
	}
}


main (int argc, char *argv[])
{
	int i, err;
	unsigned int rate;
	pid_t pid;

	start_delay = 1;
	hwparams.format = SND_PCM_FORMAT_S16_LE;
	hwparams.rate = 44100;
	hwparams.channels = 2;

        if ((err = snd_pcm_open (&capture_handle, pcm_name, SND_PCM_STREAM_CAPTURE, 0)) < 0) {
                fprintf (stderr, "cannot open audio capture device %s (%s)\n",
                         pcm_name,
                         snd_strerror(err));
                exit (1);
        }

        if ((err = snd_pcm_open (&playback_handle, pcm_name, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
                fprintf (stderr, "cannot open audio playback device %s (%s)\n",
                         pcm_name,
                         snd_strerror(err));
                exit (1);
        }

	set_params(capture_handle);
	set_params(playback_handle);

	play_buf = (u_char *)malloc(4096);
	if (play_buf == NULL) {
		perror("not enough memory\n");
		exit(1);
	}

        cap_buf = (u_char *)malloc(4096);
        if (cap_buf == NULL) {
                perror("not enough memory\n");
                exit(1);
        }

//	pipe(fd);//创建管道
	
	pid = fork();

	if(pid == -1){
		printf("Error: fork\n");
		exit(1);
	}else if(pid == 0){
		capture(argv[1]);
		snd_pcm_close(capture_handle);
		free(cap_buf);
	}else{
		playback(argv[1]);
		snd_pcm_close(playback_handle);
		free(play_buf);
	}

	exit (0);
}

